@@ -16,19 +16,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include <pyglib.h>
-
 #include "pygobject-private.h"
-#include "pygparamspec.h"
-#include "pygtype.h"
 
 /* -------------- __gtype__ objects ---------------------------- */
 
@@ -37,19 +29,12 @@ typedef struct {
     GType type;
 } PyGTypeWrapper;
 
-PYGLIB_DEFINE_TYPE("gobject.GType", PyGTypeWrapper_Type, PyGTypeWrapper);
-
-static PyObject*
-pyg_type_wrapper_richcompare(PyObject *self, PyObject *other, int op)
+static int
+pyg_type_wrapper_compare(PyGTypeWrapper *self, PyGTypeWrapper *v)
 {
-    if (Py_TYPE(self) == Py_TYPE(other) && Py_TYPE(self) == &PyGTypeWrapper_Type)
-        return _pyglib_generic_long_richcompare(((PyGTypeWrapper*)self)->type,
-                                                ((PyGTypeWrapper*)other)->type,
-                                                op);
-    else {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->type == v->type) return 0;
+    if (self->type > v->type) return -1;
+    return 1;
 }
 
 static long
@@ -65,8 +50,8 @@ pyg_type_wrapper_repr(PyGTypeWrapper *self)
     const gchar *name = g_type_name(self->type);
 
     g_snprintf(buf, sizeof(buf), "<GType %s (%lu)>",
-	       name?name:"invalid", (unsigned long int) self->type);
-    return PYGLIB_PyUnicode_FromString(buf);
+	       name?name:"invalid", self->type);
+    return PyString_FromString(buf);
 }
 
 static void
@@ -75,36 +60,12 @@ pyg_type_wrapper_dealloc(PyGTypeWrapper *self)
     PyObject_DEL(self);
 }
 
-static GQuark
-_pyg_type_key(GType type) {
-    GQuark key;
-
-    if (g_type_is_a(type, G_TYPE_INTERFACE)) {
-        key = pyginterface_type_key;
-    } else if (g_type_is_a(type, G_TYPE_ENUM)) {
-        key = pygenum_class_key;
-    } else if (g_type_is_a(type, G_TYPE_FLAGS)) {
-        key = pygflags_class_key;
-    } else if (g_type_is_a(type, G_TYPE_POINTER)) {
-        key = pygpointer_class_key;
-    } else if (g_type_is_a(type, G_TYPE_BOXED)) {
-        key = pygboxed_type_key;
-    } else {
-        key = pygobject_class_key;
-    }
-
-    return key;
-}
-
 static PyObject *
 _wrap_g_type_wrapper__get_pytype(PyGTypeWrapper *self, void *closure)
 {
-    GQuark key;
     PyObject *py_type;
-
-    key = _pyg_type_key(self->type);
-
-    py_type = g_type_get_qdata(self->type, key);
+    
+    py_type = g_type_get_qdata(self->type, pygobject_class_key);
     if (!py_type)
       py_type = Py_None;
 
@@ -112,34 +73,11 @@ _wrap_g_type_wrapper__get_pytype(PyGTypeWrapper *self, void *closure)
     return py_type;
 }
 
-static int
-_wrap_g_type_wrapper__set_pytype(PyGTypeWrapper *self, PyObject* value, void *closure)
-{
-    GQuark key;
-    PyObject *py_type;
-
-    key = _pyg_type_key(self->type);
-
-    py_type = g_type_get_qdata(self->type, key);
-    Py_CLEAR(py_type);
-    if (value == Py_None)
-	g_type_set_qdata(self->type, key, NULL);
-    else if (PyType_Check(value)) {
-	Py_INCREF(value);
-	g_type_set_qdata(self->type, key, value);
-    } else {
-	PyErr_SetString(PyExc_TypeError, "Value must be None or a type object");
-	return -1;
-    }
-
-    return 0;
-}
-
 static PyObject *
 _wrap_g_type_wrapper__get_name(PyGTypeWrapper *self, void *closure)
 {
    const char *name = g_type_name(self->type);
-   return PYGLIB_PyUnicode_FromString(name ? name : "invalid");
+   return PyString_FromString(g_strdup(name ? name : "invalid"));
 }
 
 static PyObject *
@@ -160,14 +98,14 @@ _wrap_g_type_wrapper__get_children(PyGTypeWrapper *self, void *closure)
   guint n_children, i;
   GType *children;
   PyObject *retval;
-
+    
   children = g_type_children(self->type, &n_children);
 
   retval = PyList_New(n_children);
   for (i = 0; i < n_children; i++)
       PyList_SetItem(retval, i, pyg_type_wrapper_new(children[i]));
   g_free(children);
-
+  
   return retval;
 }
 
@@ -177,25 +115,25 @@ _wrap_g_type_wrapper__get_interfaces(PyGTypeWrapper *self, void *closure)
   guint n_interfaces, i;
   GType *interfaces;
   PyObject *retval;
-
+    
   interfaces = g_type_interfaces(self->type, &n_interfaces);
 
   retval = PyList_New(n_interfaces);
   for (i = 0; i < n_interfaces; i++)
       PyList_SetItem(retval, i, pyg_type_wrapper_new(interfaces[i]));
   g_free(interfaces);
-
+  
   return retval;
 }
 
 static PyObject *
 _wrap_g_type_wrapper__get_depth(PyGTypeWrapper *self, void *closure)
 {
-  return PYGLIB_PyLong_FromLong(g_type_depth(self->type));
+  return PyInt_FromLong(g_type_depth(self->type));
 }
 
 static PyGetSetDef _PyGTypeWrapper_getsets[] = {
-    { "pytype", (getter)_wrap_g_type_wrapper__get_pytype, (setter)_wrap_g_type_wrapper__set_pytype },
+    { "pytype", (getter)_wrap_g_type_wrapper__get_pytype, (setter)0 },
     { "name",  (getter)_wrap_g_type_wrapper__get_name, (setter)0 },
     { "fundamental",  (getter)_wrap_g_type_wrapper__get_fundamental, (setter)0 },
     { "parent",  (getter)_wrap_g_type_wrapper__get_parent, (setter)0 },
@@ -264,7 +202,7 @@ _wrap_g_type_from_name(PyGTypeWrapper *_, PyObject *args)
 {
     char *type_name;
     GType type;
-
+    
     if (!PyArg_ParseTuple(args, "s:GType.from_name", &type_name))
 	return NULL;
 
@@ -287,7 +225,7 @@ _wrap_g_type_is_a(PyGTypeWrapper *self, PyObject *args)
 	return NULL;
     else if ((parent = pyg_type_from_object(gparent)) == 0)
 	return NULL;
-
+    
     return PyBool_FromLong(g_type_is_a(self->type, parent));
 }
 
@@ -305,14 +243,14 @@ static PyMethodDef _PyGTypeWrapper_methods[] = {
     { "is_a", (PyCFunction)_wrap_g_type_is_a, METH_VARARGS },
     { NULL,  0, 0 }
 };
-
+    
 static int
 pyg_type_wrapper_init(PyGTypeWrapper *self, PyObject *args, PyObject *kwargs)
 {
     static char *kwlist[] = { "object", NULL };
     PyObject *py_object;
     GType type;
-
+    
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
 				     "O:GType.__init__",
 				     kwlist, &py_object))
@@ -326,6 +264,46 @@ pyg_type_wrapper_init(PyGTypeWrapper *self, PyObject *args, PyObject *kwargs)
     return 0;
 }
 
+PyTypeObject PyGTypeWrapper_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,
+    "gobject.GType",
+    sizeof(PyGTypeWrapper),
+    0,
+    (destructor)pyg_type_wrapper_dealloc,
+    (printfunc)0,
+    (getattrfunc)0,
+    (setattrfunc)0,
+    (cmpfunc)pyg_type_wrapper_compare,
+    (reprfunc)pyg_type_wrapper_repr,
+    0,
+    0,
+    0,
+    (hashfunc)pyg_type_wrapper_hash,
+    (ternaryfunc)0,
+    (reprfunc)0,
+    (getattrofunc)0,		
+    (setattrofunc)0,		
+    0,				
+    Py_TPFLAGS_DEFAULT, 
+    NULL,
+    (traverseproc)0,		
+    (inquiry)0,
+    (richcmpfunc)0,	
+    0,             
+    (getiterfunc)0,
+    (iternextfunc)0,
+    _PyGTypeWrapper_methods,
+    0,
+    _PyGTypeWrapper_getsets,
+    NULL,		
+    NULL,		
+    (descrgetfunc)0,	
+    (descrsetfunc)0,	
+    0,                 
+    (initproc)pyg_type_wrapper_init
+};
+
 /**
  * pyg_type_wrapper_new:
  * type: a GType
@@ -349,20 +327,16 @@ pyg_type_wrapper_new(GType type)
 }
 
 /**
- * pyg_type_from_object_strict:
+ * pyg_type_from_object:
  * obj: a Python object
- * strict: if set to TRUE, raises an exception if it can't perform the
- *         conversion
  *
- * converts a python object to a GType.  If strict is set, raises an 
- * exception if it can't perform the conversion, otherwise returns
- * PY_TYPE_OBJECT.
+ * converts a python object to a GType.  Raises an exception if it
+ * can't perform the conversion.
  *
  * Returns: the corresponding GType, or 0 on error.
  */
-
 GType
-pyg_type_from_object_strict(PyObject *obj, gboolean strict)
+pyg_type_from_object(PyObject *obj)
 {
     PyObject *gtype;
     GType type;
@@ -379,7 +353,7 @@ pyg_type_from_object_strict(PyObject *obj, gboolean strict)
     if (PyType_Check(obj)) {
 	PyTypeObject *tp = (PyTypeObject *)obj;
 
-	if (tp == &PYGLIB_PyLong_Type)
+	if (tp == &PyInt_Type)
 	    return G_TYPE_INT;
 	else if (tp == &PyBool_Type)
 	    return G_TYPE_BOOLEAN;
@@ -387,20 +361,20 @@ pyg_type_from_object_strict(PyObject *obj, gboolean strict)
 	    return G_TYPE_LONG;
 	else if (tp == &PyFloat_Type)
 	    return G_TYPE_DOUBLE;
-	else if (tp == &PYGLIB_PyUnicode_Type)
+	else if (tp == &PyString_Type)
 	    return G_TYPE_STRING;
 	else if (tp == &PyBaseObject_Type)
 	    return PY_TYPE_OBJECT;
     }
 
-    if (Py_TYPE(obj) == &PyGTypeWrapper_Type) {
+    if (obj->ob_type == &PyGTypeWrapper_Type) {
 	return ((PyGTypeWrapper *)obj)->type;
     }
 
     /* handle strings */
-    if (PYGLIB_PyUnicode_Check(obj)) {
-	gchar *name = PYGLIB_PyUnicode_AsString(obj);
-
+    if (PyString_Check(obj)) {
+	gchar *name = PyString_AsString(obj);
+	
 	type = _pyg_type_from_name(name);
 	if (type != 0) {
 	    return type;
@@ -411,7 +385,7 @@ pyg_type_from_object_strict(PyObject *obj, gboolean strict)
     gtype = PyObject_GetAttrString(obj, "__gtype__");
 
     if (gtype) {
-	if (Py_TYPE(gtype) == &PyGTypeWrapper_Type) {
+	if (gtype->ob_type == &PyGTypeWrapper_Type) {
 	    type = ((PyGTypeWrapper *)gtype)->type;
 	    Py_DECREF(gtype);
 	    return type;
@@ -420,35 +394,10 @@ pyg_type_from_object_strict(PyObject *obj, gboolean strict)
     }
 
     PyErr_Clear();
-
-    /* Some API like those that take GValues can hold a python object as
-     * a pointer.  This is potentially dangerous becuase everything is 
-     * passed in as a PyObject so we can't actually type check it.  Only
-     * fallback to PY_TYPE_OBJECT if strict checking is disabled
-     */
-    if (!strict)
-        return PY_TYPE_OBJECT;
-
     PyErr_SetString(PyExc_TypeError, "could not get typecode from object");
     return 0;
 }
 
-/**
- * pyg_type_from_object:
- * obj: a Python object
- *
- * converts a python object to a GType.  Raises an exception if it
- * can't perform the conversion.
- *
- * Returns: the corresponding GType, or 0 on error.
- */
-GType
-pyg_type_from_object(PyObject *obj)
-{
-    /* Legacy call always defaults to strict type checking */
-    return pyg_type_from_object_strict(obj, TRUE);
-}
-
 /* -------------- GValue marshalling ------------------ */
 
 /**
@@ -474,29 +423,13 @@ pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
     if (!obj) {
 	*val = 0;
 	res = 0;
-    } else if (PYGLIB_PyLong_Check(obj)) {
-	*val = PYGLIB_PyLong_AsLong(obj);
+    } else if (PyInt_Check(obj)) {
+	*val = PyInt_AsLong(obj);
 	res = 0;
-
-	if (PyObject_TypeCheck(obj, &PyGEnum_Type) && ((PyGEnum *) obj)->gtype != enum_type) {
-	    g_warning("expected enumeration type %s, but got %s instead",
-		      g_type_name(enum_type),
-		      g_type_name(((PyGEnum *) obj)->gtype));
-	}
-    /* Dumb code duplication, but probably not worth it to have yet another macro. */
-    } else if (PyLong_Check(obj)) {
-	*val = PyLong_AsLong(obj);
-	res = 0;
-
-	if (PyObject_TypeCheck(obj, &PyGEnum_Type) && ((PyGEnum *) obj)->gtype != enum_type) {
-	    g_warning("expected enumeration type %s, but got %s instead",
-		      g_type_name(enum_type),
-		      g_type_name(((PyGEnum *) obj)->gtype));
-	}
-    } else if (PYGLIB_PyUnicode_Check(obj)) {
+    } else if (PyString_Check(obj)) {
 	GEnumValue *info;
-	char *str = PYGLIB_PyUnicode_AsString(obj);
-
+	char *str = PyString_AsString(obj);
+	
 	if (enum_type != G_TYPE_NONE)
 	    eclass = G_ENUM_CLASS(g_type_class_ref(enum_type));
 	else {
@@ -547,15 +480,15 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
     if (!obj) {
 	*val = 0;
 	res = 0;
-    } else if (PYGLIB_PyLong_Check(obj)) {
-	*val = PYGLIB_PyLong_AsLong(obj);
+    } else if (PyInt_Check(obj)) {
+	*val = PyInt_AsLong(obj);
 	res = 0;
     } else if (PyLong_Check(obj)) {
         *val = PyLong_AsLongLong(obj);
         res = 0;
-    } else if (PYGLIB_PyUnicode_Check(obj)) {
+    } else if (PyString_Check(obj)) {
 	GFlagsValue *info;
-	char *str = PYGLIB_PyUnicode_AsString(obj);
+	char *str = PyString_AsString(obj);
 
 	if (flag_type != G_TYPE_NONE)
 	    fclass = G_FLAGS_CLASS(g_type_class_ref(flag_type));
@@ -565,7 +498,7 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 	}
 	info = g_flags_get_value_by_name(fclass, str);
 	g_type_class_unref(fclass);
-
+	
 	if (!info)
 	    info = g_flags_get_value_by_nick(fclass, str);
 	if (info) {
@@ -591,7 +524,7 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 
 	for (i = 0; i < len; i++) {
 	    PyObject *item = PyTuple_GetItem(obj, i);
-	    char *str = PYGLIB_PyUnicode_AsString(item);
+	    char *str = PyString_AsString(item);
 	    GFlagsValue *info = g_flags_get_value_by_name(fclass, str);
 
 	    if (!info)
@@ -678,7 +611,7 @@ pyg_value_array_from_pyobject(GValue *value,
 
     if (pspec && pspec->fixed_n_elements > 0 && len != pspec->fixed_n_elements)
 	return -1;
-
+	    
     value_array = g_value_array_new(len);
 
     for (i = 0; i < len; ++i) {
@@ -695,10 +628,8 @@ pyg_value_array_from_pyobject(GValue *value,
 
 	if (pspec && pspec->element_spec)
 	    type = G_PARAM_SPEC_VALUE_TYPE(pspec->element_spec);
-	else if (item == Py_None)
-	    type = G_TYPE_POINTER; /* store None as NULL */
 	else {
-	    type = pyg_type_from_object((PyObject*)Py_TYPE(item));
+	    type = pyg_type_from_object((PyObject *) item->ob_type);
 	    if (! type) {
 		PyErr_Clear();
 		g_value_array_free(value_array);
@@ -748,116 +679,107 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
     case G_TYPE_INTERFACE:
 	/* we only handle interface types that have a GObject prereq */
 	if (g_type_is_a(G_VALUE_TYPE(value), G_TYPE_OBJECT)) {
-	    if (obj == Py_None)
-		g_value_set_object(value, NULL);
-	    else {
-		if (!PyObject_TypeCheck(obj, &PyGObject_Type)) {
-		    return -1;
-		}
-		if (!G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
-						G_VALUE_TYPE(value))) {
-		    return -1;
-		}
-		g_value_set_object(value, pygobject_get(obj));
+	    if (!PyObject_TypeCheck(obj, &PyGObject_Type)) {
+		return -1;
 	    }
+	    if (!G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
+					    G_VALUE_TYPE(value))) {
+		return -1;
+	    }
+	    g_value_set_object(value, pygobject_get(obj));
 	} else {
 	    return -1;
 	}
 	break;
     case G_TYPE_CHAR:
-#if PY_VERSION_HEX < 0x03000000
-	if (PyString_Check(obj)) {
-	    g_value_set_char(value, PyString_AsString(obj)[0]);
-	} else
-#endif
-	if (PyUnicode_Check(obj)) {
-	    tmp = PyUnicode_AsUTF8String(obj);
-	    g_value_set_char(value, PYGLIB_PyBytes_AsString(tmp)[0]);
-	    Py_DECREF(tmp);
-	} else {
+	if ((tmp = PyObject_Str(obj)))
+	    g_value_set_char(value, PyString_AsString(tmp)[0]);
+	else {
 	    PyErr_Clear();
 	    return -1;
 	}
-
+	Py_DECREF(tmp);
 	break;
     case G_TYPE_UCHAR:
-	if (PYGLIB_PyLong_Check(obj)) {
-	    glong val;
-	    val = PYGLIB_PyLong_AsLong(obj);
-	    if (val >= 0 && val <= 255)
-	      g_value_set_uchar(value, (guchar)PYGLIB_PyLong_AsLong (obj));
-	    else
-	      return -1;
-#if PY_VERSION_HEX < 0x03000000
-	} else if (PyString_Check(obj)) {
-	    g_value_set_uchar(value, PyString_AsString(obj)[0]);
-#endif
-	} else if (PyUnicode_Check(obj)) {
-	    tmp = PyUnicode_AsUTF8String(obj);
-	    g_value_set_uchar(value, PYGLIB_PyBytes_AsString(tmp)[0]);
-	    Py_DECREF(tmp);
-	} else {
+	if ((tmp = PyObject_Str(obj)))
+	    g_value_set_char(value, PyString_AsString(tmp)[0]);
+	else {
 	    PyErr_Clear();
 	    return -1;
 	}
+	Py_DECREF(tmp);
 	break;
     case G_TYPE_BOOLEAN:
 	g_value_set_boolean(value, PyObject_IsTrue(obj));
 	break;
     case G_TYPE_INT:
-	g_value_set_int(value, PYGLIB_PyLong_AsLong(obj));
+	g_value_set_int(value, PyInt_AsLong(obj));
+	if (PyErr_Occurred()) {
+	    g_value_unset(value);
+	    PyErr_Clear();
+	    return -1;
+	}
 	break;
     case G_TYPE_UINT:
 	{
-	    if (PYGLIB_PyLong_Check(obj)) {
+	    if (PyInt_Check(obj)) {
 		glong val;
 
-		val = PYGLIB_PyLong_AsLong(obj);
+		val = PyInt_AsLong(obj);
 		if (val >= 0 && val <= G_MAXUINT)
 		    g_value_set_uint(value, (guint)val);
 		else
 		    return -1;
 	    } else {
 		g_value_set_uint(value, PyLong_AsUnsignedLong(obj));
+		if (PyErr_Occurred()) {
+		    g_value_unset(value);
+		    PyErr_Clear();
+		    return -1;
+		}
 	    }
 	}
 	break;
     case G_TYPE_LONG:
-	g_value_set_long(value, PYGLIB_PyLong_AsLong(obj));
+	g_value_set_long(value, PyInt_AsLong(obj));
+	if (PyErr_Occurred()) {
+	    g_value_unset(value);
+	    PyErr_Clear();
+	    return -1;
+	}
 	break;
     case G_TYPE_ULONG:
-#if PY_VERSION_HEX < 0x03000000
-	if (PyInt_Check(obj)) {
-            long val;
+	{
+	    if (PyInt_Check(obj)) {
+		glong val;
 
-            val = PYGLIB_PyLong_AsLong(obj);
-            if (val < 0) {
-                PyErr_SetString(PyExc_OverflowError, "negative value not allowed for uint64 property");
-                return -1;
-            }
-            g_value_set_ulong(value, (gulong)val);
-        } else
-#endif
-        if (PyLong_Check(obj))
-	    g_value_set_ulong(value, PyLong_AsUnsignedLong(obj));
-        else
-            return -1;
-        break;
+		val = PyInt_AsLong(obj);
+		if (val >= 0)
+		    g_value_set_ulong(value, (gulong)val);
+		else
+		    return -1;
+	    } else {
+		g_value_set_ulong(value, PyLong_AsUnsignedLong(obj));
+		if (PyErr_Occurred()) {
+		    g_value_unset(value);
+		    PyErr_Clear();
+		    return -1;
+		}
+	    }
+	}
+	break;
     case G_TYPE_INT64:
 	g_value_set_int64(value, PyLong_AsLongLong(obj));
+	if (PyErr_Occurred()) {
+	    g_value_unset(value);
+	    PyErr_Clear();
+	    return -1;
+	}
 	break;
     case G_TYPE_UINT64:
-#if PY_VERSION_HEX < 0x03000000
-        if (PyInt_Check(obj)) {
-            long v = PyInt_AsLong(obj);
-            if (v < 0) {
-                PyErr_SetString(PyExc_OverflowError, "negative value not allowed for uint64 property");
-                return -1;
-            }
-            g_value_set_uint64(value, v);
-        } else
-#endif
-        if (PyLong_Check(obj))
+        if (PyInt_Check(obj))
+            g_value_set_uint64(value, PyInt_AsLong(obj));
+        else if (PyLong_Check(obj))
             g_value_set_uint64(value, PyLong_AsUnsignedLongLong(obj));
         else
             return -1;
@@ -884,34 +806,29 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	break;
     case G_TYPE_FLOAT:
 	g_value_set_float(value, PyFloat_AsDouble(obj));
+	if (PyErr_Occurred()) {
+	    g_value_unset(value);
+	    PyErr_Clear();
+	    return -1;
+	}
 	break;
     case G_TYPE_DOUBLE:
 	g_value_set_double(value, PyFloat_AsDouble(obj));
+	if (PyErr_Occurred()) {
+	    g_value_unset(value);
+	    PyErr_Clear();
+	    return -1;
+	}
 	break;
     case G_TYPE_STRING:
-	if (obj == Py_None) {
+	if (obj == Py_None)
 	    g_value_set_string(value, NULL);
+	else if ((tmp = PyObject_Str(obj))) {
+	    g_value_set_string(value, PyString_AsString(tmp));
+	    Py_DECREF(tmp);
 	} else {
-	    PyObject* tmp_str = PyObject_Str(obj);
-	    if (tmp_str == NULL) {
-	        PyErr_Clear();
-	        if (PyUnicode_Check(obj)) {
-	            tmp = PyUnicode_AsUTF8String(obj);
-	            g_value_set_string(value, PYGLIB_PyBytes_AsString(tmp));
-	            Py_DECREF(tmp);
-	        } else {
-	            return -1;
-	        }
-	    } else {
-#if PY_VERSION_HEX < 0x03000000
-	       g_value_set_string(value, PyString_AsString(tmp_str));
-#else
-	       tmp = PyUnicode_AsUTF8String(tmp_str);
-	       g_value_set_string(value, PyBytes_AsString(tmp));
-	       Py_DECREF(tmp);
-#endif
-	    }
-	    Py_XDECREF(tmp_str);
+	    PyErr_Clear();
+	    return -1;
 	}
 	break;
     case G_TYPE_POINTER:
@@ -920,8 +837,8 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	else if (PyObject_TypeCheck(obj, &PyGPointer_Type) &&
 		   G_VALUE_HOLDS(value, ((PyGPointer *)obj)->gtype))
 	    g_value_set_pointer(value, pyg_pointer_get(obj, gpointer));
-	else if (PYGLIB_CPointer_Check(obj))
-	    g_value_set_pointer(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+	else if (PyCObject_Check(obj))
+	    g_value_set_pointer(value, PyCObject_AsVoidPtr(obj));
 	else
 	    return -1;
 	break;
@@ -930,51 +847,36 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 
 	if (obj == Py_None)
 	    g_value_set_boxed(value, NULL);
-	else if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
+	if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
 	    g_value_set_boxed(value, obj);
 	else if (PyObject_TypeCheck(obj, &PyGBoxed_Type) &&
 		   G_VALUE_HOLDS(value, ((PyGBoxed *)obj)->gtype))
 	    g_value_set_boxed(value, pyg_boxed_get(obj, gpointer));
-        else if (G_VALUE_HOLDS(value, G_TYPE_VALUE)) {
-            GType type;
-            GValue *n_value;
-
-            type = pyg_type_from_object((PyObject*)Py_TYPE(obj));
-            if (G_UNLIKELY (! type)) {
-                PyErr_Clear();
-                return -1;
-            }
-            n_value = g_new0 (GValue, 1);
-            g_value_init (n_value, type);
-            g_value_take_boxed (value, n_value);
-            return pyg_value_from_pyobject (n_value, obj);
-        }
-        else if (PySequence_Check(obj) &&
+	else if (PySequence_Check(obj) &&
 		   G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY))
 	    return pyg_value_array_from_pyobject(value, obj, NULL);
-	else if (PYGLIB_PyUnicode_Check(obj) &&
+	else if (PyString_Check(obj) &&
                  G_VALUE_HOLDS(value, G_TYPE_GSTRING)) {
             GString *string;
             char *buffer;
-            Py_ssize_t len;
-            if (PYGLIB_PyUnicode_AsStringAndSize(obj, &buffer, &len))
+            int len;
+            if (PyString_AsStringAndSize(obj, &buffer, &len))
                 return -1;
             string = g_string_new_len(buffer, len);
 	    g_value_set_boxed(value, string);
-	    g_string_free (string, TRUE);
             break;
         }
 	else if ((bm = pyg_type_lookup(G_VALUE_TYPE(value))) != NULL)
 	    return bm->tovalue(value, obj);
-	else if (PYGLIB_CPointer_Check(obj))
-	    g_value_set_boxed(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+	else if (PyCObject_Check(obj))
+	    g_value_set_boxed(value, PyCObject_AsVoidPtr(obj));
 	else
 	    return -1;
 	break;
     }
     case G_TYPE_PARAM:
 	if (PyGParamSpec_Check(obj))
-	    g_value_set_param(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+	    g_value_set_param(value, PyCObject_AsVoidPtr(obj));
 	else
 	    return -1;
 	break;
@@ -996,11 +898,6 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	    break;
 	}
     }
-    if (PyErr_Occurred()) {
-        g_value_unset(value);
-        PyErr_Clear();
-        return -1;
-    }
     return 0;
 }
 
@@ -1022,22 +919,22 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
     switch (G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(value))) {
     case G_TYPE_INTERFACE:
 	if (g_type_is_a(G_VALUE_TYPE(value), G_TYPE_OBJECT))
-	    return pygobject_new_sunk(g_value_get_object(value));
+	    return pygobject_new(g_value_get_object(value));
 	else
 	    break;
     case G_TYPE_CHAR: {
 	gint8 val = g_value_get_char(value);
-	return PYGLIB_PyUnicode_FromStringAndSize((char *)&val, 1);
+	return PyString_FromStringAndSize((char *)&val, 1);
     }
     case G_TYPE_UCHAR: {
 	guint8 val = g_value_get_uchar(value);
-	return PYGLIB_PyBytes_FromStringAndSize((char *)&val, 1);
+	return PyString_FromStringAndSize((char *)&val, 1);
     }
     case G_TYPE_BOOLEAN: {
 	return PyBool_FromLong(g_value_get_boolean(value));
     }
     case G_TYPE_INT:
-	return PYGLIB_PyLong_FromLong(g_value_get_int(value));
+	return PyInt_FromLong(g_value_get_int(value));
     case G_TYPE_UINT:
 	{
 	    /* in Python, the Int object is backed by a long.  If a
@@ -1045,19 +942,19 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	       an Int.  Otherwise, use a Long object to avoid overflow.
 	       This matches the ULongArg behavior in codegen/argtypes.h */
 #if (G_MAXUINT <= G_MAXLONG)
-	    return PYGLIB_PyLong_FromLong((glong) g_value_get_uint(value));
+	    return PyInt_FromLong((glong) g_value_get_uint(value));
 #else
 	    return PyLong_FromUnsignedLong((gulong) g_value_get_uint(value));
 #endif
 	}
     case G_TYPE_LONG:
-	return PYGLIB_PyLong_FromLong(g_value_get_long(value));
+	return PyInt_FromLong(g_value_get_long(value));
     case G_TYPE_ULONG:
 	{
 	    gulong val = g_value_get_ulong(value);
 
 	    if (val <= G_MAXLONG)
-		return PYGLIB_PyLong_FromLong((glong) val);
+		return PyInt_FromLong((glong) val);
 	    else
 		return PyLong_FromUnsignedLong(val);
 	}
@@ -1066,7 +963,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    gint64 val = g_value_get_int64(value);
 
 	    if (G_MINLONG <= val && val <= G_MAXLONG)
-		return PYGLIB_PyLong_FromLong((glong) val);
+		return PyInt_FromLong((glong) val);
 	    else
 		return PyLong_FromLongLong(val);
 	}
@@ -1075,7 +972,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    guint64 val = g_value_get_uint64(value);
 
 	    if (val <= G_MAXLONG)
-		return PYGLIB_PyLong_FromLong((glong) val);
+		return PyInt_FromLong((glong) val);
 	    else
 		return PyLong_FromUnsignedLongLong(val);
 	}
@@ -1092,7 +989,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    const gchar *str = g_value_get_string(value);
 
 	    if (str)
-		return PYGLIB_PyUnicode_FromString(str);
+		return PyString_FromString(str);
 	    Py_INCREF(Py_None);
 	    return Py_None;
 	}
@@ -1109,10 +1006,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 		return Py_None;
 	    }
 	    return ret;
-        } else if (G_VALUE_HOLDS(value, G_TYPE_VALUE)) {
-            GValue *n_value = g_value_get_boxed (value);
-            return pyg_value_as_pyobject(n_value, copy_boxed);
-        } else if (G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY)) {
+	} else if (G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY)) {
 	    GValueArray *array = (GValueArray *) g_value_get_boxed(value);
 	    PyObject *ret = PyList_New(array->n_values);
 	    int i;
@@ -1122,9 +1016,9 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 	    return ret;
 	} else if (G_VALUE_HOLDS(value, G_TYPE_GSTRING)) {
 	    GString *string = (GString *) g_value_get_boxed(value);
-	    PyObject *ret = PYGLIB_PyUnicode_FromStringAndSize(string->str, string->len);
+	    PyObject *ret = PyString_FromStringAndSize(string->str, string->len);
 	    return ret;
-	}
+	}	    
 	bm = pyg_type_lookup(G_VALUE_TYPE(value));
 	if (bm) {
 	    return bm->fromvalue(value);
@@ -1140,7 +1034,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
     case G_TYPE_PARAM:
 	return pyg_param_spec_new(g_value_get_param(value));
     case G_TYPE_OBJECT:
-	return pygobject_new_sunk(g_value_get_object(value));
+	return pygobject_new(g_value_get_object(value));
     default:
 	{
 	    PyGTypeMarshal *bm;
@@ -1163,11 +1057,11 @@ pyg_closure_invalidate(gpointer data, GClosure *closure)
     PyGClosure *pc = (PyGClosure *)closure;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = pyg_gil_state_ensure();
     Py_XDECREF(pc->callback);
     Py_XDECREF(pc->extra_args);
     Py_XDECREF(pc->swap_data);
-    pyglib_gil_state_release(state);
+    pyg_gil_state_release(state);
 
     pc->callback = NULL;
     pc->extra_args = NULL;
@@ -1187,7 +1081,7 @@ pyg_closure_marshal(GClosure *closure,
     PyObject *params, *ret;
     guint i;
 
-    state = pyglib_gil_state_ensure();
+    state = pyg_gil_state_ensure();
 
     /* construct Python tuple for the parameter values */
     params = PyTuple_New(n_param_values);
@@ -1221,21 +1115,21 @@ pyg_closure_marshal(GClosure *closure,
 	    PyErr_Print();
 	goto out;
     }
-
+    
     if (return_value && pyg_value_from_pyobject(return_value, ret) != 0) {
 	PyErr_SetString(PyExc_TypeError,
 			"can't convert return value to desired type");
-
+	
 	if (pc->exception_handler)
 	    pc->exception_handler(return_value, n_param_values, param_values);
 	else
 	    PyErr_Print();
     }
     Py_DECREF(ret);
-
+    
  out:
     Py_DECREF(params);
-    pyglib_gil_state_release(state);
+    pyg_gil_state_release(state);
 }
 
 /**
@@ -1284,7 +1178,7 @@ pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data)
  * @handler: the handler to call when an exception occurs or NULL for none
  *
  * Sets the handler to call when an exception occurs during closure invocation.
- * The handler is responsible for providing a proper return value to the
+ * The handler is responsible for providing a proper return value to the 
  * closure invocation. If @handler is %NULL, the default handler will be used.
  * The default handler prints the exception to stderr and doesn't touch the
  * closure's return value.
@@ -1294,7 +1188,7 @@ pyg_closure_set_exception_handler(GClosure *closure,
 				  PyClosureExceptionHandler handler)
 {
     PyGClosure *pygclosure;
-
+    
     g_return_if_fail(closure != NULL);
 
     pygclosure = (PyGClosure *)closure;
@@ -1328,15 +1222,15 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     PyObject *params, *ret;
     guint i, len;
 
-    state = pyglib_gil_state_ensure();
-
+    state = pyg_gil_state_ensure();
+    
     g_return_if_fail(invocation_hint != NULL);
     /* get the object passed as the first argument to the closure */
     object = g_value_get_object(&param_values[0]);
     g_return_if_fail(object != NULL && G_IS_OBJECT(object));
 
     /* get the wrapper for this object */
-    object_wrapper = pygobject_new_sunk(object);
+    object_wrapper = pygobject_new(object);
     g_return_if_fail(object_wrapper != NULL);
 
     /* construct method name for this class closure */
@@ -1354,7 +1248,7 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     if (!method) {
 	PyErr_Clear();
 	Py_DECREF(object_wrapper);
-	pyglib_gil_state_release(state);
+	pyg_gil_state_release(state);
 	return;
     }
     Py_DECREF(object_wrapper);
@@ -1368,17 +1262,17 @@ pyg_signal_class_closure_marshal(GClosure *closure,
 	/* error condition */
 	if (!item) {
 	    Py_DECREF(params);
-	    pyglib_gil_state_release(state);
+	    pyg_gil_state_release(state);
 	    return;
 	}
 	PyTuple_SetItem(params, i - 1, item);
     }
 
     ret = PyObject_CallObject(method, params);
-
+    
     /* Copy boxed values if others ref them, this needs to be done regardless of
        exception status. */
-    len = PyTuple_Size(params);
+    len = PyTuple_Size(params);    
     for (i = 0; i < len; i++) {
 	PyObject *item = PyTuple_GetItem(params, i);
 	if (item != NULL && PyObject_TypeCheck(item, &PyGBoxed_Type)
@@ -1395,7 +1289,7 @@ pyg_signal_class_closure_marshal(GClosure *closure,
 	PyErr_Print();
 	Py_DECREF(method);
 	Py_DECREF(params);
-	pyglib_gil_state_release(state);
+	pyg_gil_state_release(state);
 	return;
     }
     Py_DECREF(method);
@@ -1403,7 +1297,7 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     if (return_value)
 	pyg_value_from_pyobject(return_value, ret);
     Py_DECREF(ret);
-    pyglib_gil_state_release(state);
+    pyg_gil_state_release(state);
 }
 
 /**
@@ -1434,18 +1328,13 @@ GClosure *
 gclosure_from_pyfunc(PyGObject *object, PyObject *func)
 {
     GSList *l;
-    PyGObjectData *inst_data;
-    inst_data = pyg_object_peek_inst_data(object->obj);
-    if (inst_data) {
-        for (l = inst_data->closures; l; l = l->next) {
-            PyGClosure *pyclosure = l->data;
-            int res = PyObject_RichCompareBool(pyclosure->callback, func, Py_EQ);
-            if (res == -1) {
-                PyErr_Clear(); // Is there anything else to do?
-            } else if (res) {
-                return (GClosure*)pyclosure;
-            }
-        }
+
+    for (l = object->closures; l; l = l->next) {
+	PyGClosure *pyclosure = l->data;
+	if (PyFunction_GetClosure(pyclosure->callback) ==
+		PyFunction_GetClosure(func)) {
+	    return (GClosure*)pyclosure;
+	}
     }
     return NULL;
 }
@@ -1528,7 +1417,7 @@ add_property_docs(GType gtype, GString *string)
 			       g_type_name(props[i]->value_type),
 			       g_param_spec_get_nick(props[i]));
 
-	/* g_string_append_printf crashes on win32 if the third
+	/* g_string_append_printf crashes on win32 if the third 
 	   argument is NULL. */
 	blurb=g_param_spec_get_blurb(props[i]);
 	if (blurb)
@@ -1578,7 +1467,7 @@ object_doc_descr_get(PyObject *self, PyObject *obj, PyObject *type)
             g_array_append_val(parents, parent);
             parent = g_type_next_base(gtype, parent);
         }
-
+        
         for (iparent = parents->len - 1; iparent >= 0; --iparent) {
 	    GType *interfaces;
 	    guint n_interfaces, i;
@@ -1596,12 +1485,48 @@ object_doc_descr_get(PyObject *self, PyObject *obj, PyObject *type)
         g_array_free(parents, TRUE);
     }
 
-    pystring = PYGLIB_PyUnicode_FromStringAndSize(string->str, string->len);
+    pystring = PyString_FromStringAndSize(string->str, string->len);
     g_string_free(string, TRUE);
     return pystring;
 }
 
-PYGLIB_DEFINE_TYPE("gobject.GObject.__doc__", PyGObjectDoc_Type, PyObject);
+static PyTypeObject PyGObjectDoc_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,
+    "gobject.GObject__doc__",
+    sizeof(PyObject),
+    0,
+    (destructor)object_doc_dealloc,
+    (printfunc)0,
+    (getattrfunc)0,
+    (setattrfunc)0,
+    (cmpfunc)0,
+    (reprfunc)0,
+    0,
+    0,
+    0,
+    (hashfunc)0,
+    (ternaryfunc)0,
+    (reprfunc)0,
+    (getattrofunc)0,
+    (setattrofunc)0,
+    0,
+    Py_TPFLAGS_DEFAULT,
+    NULL,
+    (traverseproc)0,
+    (inquiry)0,
+    (richcmpfunc)0,
+    0,
+    (getiterfunc)0,
+    (iternextfunc)0,
+    0,
+    0,
+    0,
+    (PyTypeObject *)0,
+    (PyObject *)0,
+    (descrgetfunc)object_doc_descr_get,
+    (descrsetfunc)0
+};
 
 /**
  * pyg_object_descr_doc_get:
@@ -1618,7 +1543,7 @@ pyg_object_descr_doc_get(void)
     static PyObject *doc_descr = NULL;
 
     if (!doc_descr) {
-	Py_TYPE(&PyGObjectDoc_Type) = &PyType_Type;
+	PyGObjectDoc_Type.ob_type = &PyType_Type;
 	if (PyType_Ready(&PyGObjectDoc_Type))
 	    return NULL;
 
@@ -1646,7 +1571,7 @@ int pyg_pyobj_to_unichar_conv(PyObject* py_obj, void* ptr)
     gunichar* u = ptr;
     const Py_UNICODE* uni_buffer;
     PyObject* tmp_uni = NULL;
-
+    
     if (PyUnicode_Check(py_obj)) {
 	tmp_uni = py_obj;
 	Py_INCREF(tmp_uni);
@@ -1656,7 +1581,7 @@ int pyg_pyobj_to_unichar_conv(PyObject* py_obj, void* ptr)
 	if (tmp_uni == NULL)
 	    goto failure;
     }
-
+    
     if ( PyUnicode_GetSize(tmp_uni) != 1) {
 	PyErr_SetString(PyExc_ValueError, "unicode character value must be 1 character uniode string");
 	goto failure;
@@ -1665,24 +1590,24 @@ int pyg_pyobj_to_unichar_conv(PyObject* py_obj, void* ptr)
     if ( uni_buffer == NULL)
 	goto failure;
     *u = uni_buffer[0];
-
+    
     Py_DECREF(tmp_uni);
     return 1;
-
+    
   failure:
     Py_XDECREF(tmp_uni);
     return 0;
 }
 
 
-int
-pyg_param_gvalue_from_pyobject(GValue* value,
-                               PyObject* py_obj,
+int 
+pyg_param_gvalue_from_pyobject(GValue* value, 
+                               PyObject* py_obj, 
 			       const GParamSpec* pspec)
 {
     if (G_IS_PARAM_SPEC_UNICHAR(pspec)) {
 	gunichar u;
-
+	
 	if (!pyg_pyobj_to_unichar_conv(py_obj, &u)) {
 	    PyErr_Clear();
 	    return -1;
@@ -1698,15 +1623,15 @@ pyg_param_gvalue_from_pyobject(GValue* value,
     }
 }
 
-PyObject*
+PyObject* 
 pyg_param_gvalue_as_pyobject(const GValue* gvalue,
-                             gboolean copy_boxed,
+                             gboolean copy_boxed, 
 			     const GParamSpec* pspec)
 {
     if (G_IS_PARAM_SPEC_UNICHAR(pspec)) {
 	gunichar u;
 	Py_UNICODE uni_buffer[2] = { 0, 0 };
-
+	
 	u = g_value_get_uint(gvalue);
 	uni_buffer[0] = u;
 	return PyUnicode_FromUnicode(uni_buffer, 1);
@@ -1727,7 +1652,7 @@ typedef struct {
     gpointer data;
 } CustomTypeData;
 
-void
+void 
 pyg_type_register_custom_callback(const gchar *typename,
 				  PyGTypeRegistrationFunction callback,
 				  gpointer user_data)
@@ -1752,7 +1677,7 @@ pyg_type_get_custom(const gchar *name)
 {
     CustomTypeData *data;
     PyTypeObject *retval;
-
+    
     if (!custom_type_registration)
 	return NULL;
 
@@ -1777,67 +1702,7 @@ _pyg_type_from_name(const gchar *name)
 	pyg_type_get_custom(name);
 	type = g_type_from_name(name);
     }
-
+    
     return type;
 }
 
-static PyObject *
-_pyg_strv_from_gvalue(const GValue *value)
-{
-    gchar    **argv = (gchar **) g_value_get_boxed(value);
-    int        argc = 0, i;
-    PyObject  *py_argv;
-
-    if (argv) {
-        while (argv[argc])
-            argc++;
-    }
-    py_argv = PyList_New(argc);
-    for (i = 0; i < argc; ++i)
-	PyList_SET_ITEM(py_argv, i, PYGLIB_PyUnicode_FromString(argv[i]));
-    return py_argv;
-}
-
-static int
-_pyg_strv_to_gvalue(GValue *value, PyObject *obj)
-{
-    Py_ssize_t argc, i;
-    gchar **argv;
-
-    if (!(PyTuple_Check(obj) || PyList_Check(obj)))
-        return -1;
-
-    argc = PySequence_Length(obj);
-    for (i = 0; i < argc; ++i)
-	if (!PYGLIB_PyUnicode_Check(PySequence_Fast_GET_ITEM(obj, i)))
-	    return -1;
-    argv = g_new(gchar *, argc + 1);
-    for (i = 0; i < argc; ++i)
-	argv[i] = g_strdup(PYGLIB_PyUnicode_AsString(PySequence_Fast_GET_ITEM(obj, i)));
-    argv[i] = NULL;
-    g_value_take_boxed(value, argv);
-    return 0;
-}
-
-void
-pygobject_type_register_types(PyObject *d)
-{
-    PyGTypeWrapper_Type.tp_dealloc = (destructor)pyg_type_wrapper_dealloc;
-    PyGTypeWrapper_Type.tp_richcompare = pyg_type_wrapper_richcompare;
-    PyGTypeWrapper_Type.tp_repr = (reprfunc)pyg_type_wrapper_repr;
-    PyGTypeWrapper_Type.tp_hash = (hashfunc)pyg_type_wrapper_hash;
-    PyGTypeWrapper_Type.tp_flags = Py_TPFLAGS_DEFAULT;
-    PyGTypeWrapper_Type.tp_methods = _PyGTypeWrapper_methods;
-    PyGTypeWrapper_Type.tp_getset = _PyGTypeWrapper_getsets;
-    PyGTypeWrapper_Type.tp_init = (initproc)pyg_type_wrapper_init;
-    PYGLIB_REGISTER_TYPE(d, PyGTypeWrapper_Type, "GType");
-
-    /* This type lazily registered in pyg_object_descr_doc_get */
-    PyGObjectDoc_Type.tp_dealloc = (destructor)object_doc_dealloc;
-    PyGObjectDoc_Type.tp_flags = Py_TPFLAGS_DEFAULT;
-    PyGObjectDoc_Type.tp_descr_get = (descrgetfunc)object_doc_descr_get;
-
-    pyg_register_gtype_custom(G_TYPE_STRV,
-			      _pyg_strv_from_gvalue,
-			      _pyg_strv_to_gvalue);
-}
